@@ -686,6 +686,9 @@ static struct cmd_results *cmd_move_workspace(int argc, char **argv) {
 	arrange_output(old_output);
 	arrange_output(new_output);
 
+	struct sway_seat *seat = config->handler_context.seat;
+	seat_consider_warp_to_focus(seat);
+
 	return cmd_results_new(CMD_SUCCESS, NULL);
 }
 
@@ -788,15 +791,15 @@ static struct cmd_results *cmd_move_to_position_pointer(
 	struct wlr_output *output = wlr_output_layout_output_at(
 			root->output_layout, cursor->x, cursor->y);
 	if (output) {
-		struct wlr_box *box =
-				wlr_output_layout_get_box(root->output_layout, output);
-		lx = fmax(lx, box->x);
-		ly = fmax(ly, box->y);
-		if (lx + container->pending.width > box->x + box->width) {
-			lx = box->x + box->width - container->pending.width;
+		struct wlr_box box;
+		wlr_output_layout_get_box(root->output_layout, output, &box);
+		lx = fmax(lx, box.x);
+		ly = fmax(ly, box.y);
+		if (lx + container->pending.width > box.x + box.width) {
+			lx = box.x + box.width - container->pending.width;
 		}
-		if (ly + container->pending.height > box->y + box->height) {
-			ly = box->y + box->height - container->pending.height;
+		if (ly + container->pending.height > box.y + box.height) {
+			ly = box.y + box.height - container->pending.height;
 		}
 	}
 
